import * as fs from 'fs'
import path from 'path';
import ConsoleLogger from './log.utils';
import { ClientObject, TransportServiceInterface } from '../interface/interface';
const console: ConsoleLogger = new ConsoleLogger(`GeneralUtils`, ['util'])
// Check if filename exists. Return profile information if there's any
export async function checkOwnClientInfo(filename?: string): Promise<{ id: string, clientId: string }> {
    return new Promise((resolve, reject) => {
        // Check if the file exists
        let filePath = process.env.FolderPath as string
        if (fs.existsSync(`${filePath}${filename}.json`)) {
            try {
                // Read the file contents
                const fileData = fs.readFileSync(filePath + `${filename}.json`, 'utf8');

                // If the file is empty, return an error
                if (fileData.trim() === "") {
                    throw new Error("File is empty");
                }

                // Parse and return the data if present
                const jsonData: { id: string, clientId: string } = JSON.parse(fileData);
                resolve(jsonData)

            } catch (err) {
                // Handle parsing errors or other file-related errors
                console.log({ message: "Error reading or parsing file:", details: err })
                reject('');
            }
        } else {
            console.log({ message: `File does not exist. Id: ${filename}` })
            reject('');
        }
    })
}

export async function checkIfClientExists(id: string): Promise<any> {
    return new Promise((resolve, reject) => {
        try {
            // Check if the file exists
            let filePath = process.env.FolderPath as string + 'clients.json'
            if (!fs.existsSync(filePath)) {
                console.log({ message: `File does not exist.` })
                reject('File does not exist');
            }

            // Read and parse the data
            const fileContent = fs.readFileSync(filePath, 'utf-8');
            const data: any[] = JSON.parse(fileContent);

            // Check if an details with the given id exists
            let obj = data.find(entry => entry.id === id);

            if (obj) {
                console.log({ message: `Client with ID ${id} exists.` })
            } else {
                console.log({ message: `Client with ID ${id} does not exist.` })
            }
            resolve(obj);
        } catch (error) {
            reject(`Error reading the file`)
        }
    })
}

export async function writeFile(data: any, filename: string): Promise<boolean> {
    return new Promise((resolve, reject) => {
        // Ensure the folder exists
        const folderPath = process.env.FolderPath as string
        console.log({ message: folderPath + (data.clientId ?? `undefined client Id`) })
        // const folderPath = path.join(__dirname, folder);
        if (!fs.existsSync(folderPath)) {
            fs.mkdirSync(folderPath, { recursive: true }); // Create folder if it doesn't exist
        } else {
            console.log({ message: 'Folder already exist' })
        }

        // Construct the full file path (include the folder)
        const filePath = path.join(folderPath, `${filename}.json`);

        // Write JSON data to a file
        fs.writeFile(filePath, JSON.stringify(data, null, 2), (err) => {
            if (err) {
                console.log({ message: 'Error writing file', details: err });
                reject(false);
            } else {
                console.log({ message: 'File has been written', details: filePath });
                resolve(true);
            }
        });
    });
}

/* For Internal Usage only. Temporary serve as a way for server to keep track of clients. To be replaced in the future with better alternatives. */
export function addClientToDB(entry: any, filename: string): void {
    try {
        let data: any[] = [];
        let filePath = process.env.FolderPath as string + `${filename}.json`
        // Check if the file exists and load existing data
        if (fs.existsSync(filePath)) {
            const fileContent = fs.readFileSync(filePath, 'utf-8');
            data = JSON.parse(fileContent);
        }

        // Append the new details to the array
        data.push({
            id: entry.clientId,
            dateCreated: entry.dateCreated,
        });

        // Write the updated array back to the file
        fs.writeFileSync(filePath, JSON.stringify(data, null, 2), 'utf-8');
        console.log({ message: `Entry added successfully.` })
    } catch (error) {
        console.log({ message: 'Error writing to file:', details: error })
    }
}

